from .Token import TokenService
from .Application import ApplicationService
from .Airtime import AirtimeService
from .SMS import SMSService
from .Voice import VoiceService
from .MobileData import MobileDataService
from .Insights import InsightService
from .Whatsapp import WhatsappService

SMS = None
Airtime = None
Voice = None
Application = None
Token = None
MobileData = None
Insights = None
Whatsapp = None


def initialize(username, api_key):
    if username is None or api_key is None:
        raise RuntimeError("Please check if your username and api key have been set.")

    globals()["SMS"] = SMSService(username, api_key)
    globals()["Airtime"] = AirtimeService(username, api_key)
    globals()["Voice"] = VoiceService(username, api_key)
    globals()["Application"] = ApplicationService(username, api_key)
    globals()["Token"] = TokenService(username, api_key)
    globals()["MobileData"] = MobileDataService(username, api_key)
    globals()["Insights"] = InsightService(username, api_key)
    globals()["Whatsapp"] = WhatsappService(username, api_key)
    # globals()['USSD'] = USSDService(username, api_key)



"""
Insights

fetchApplicationData(): Fetch app info i.e. balance.
"""

import africastalking
import unittest
import responses
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Insights


class TestInsightsService(unittest.TestCase):
    @responses.activate
    def test_sim_swap_state(self):
        responses.add(
            responses.POST,
            "https://insights.africastalking.com/v1/sim-swap",
            json={
                "transactionId": "a4f740c9-b0e4-4e89-a55d-8244d77383d2",
                "status": "Processed",
                "totalCost": {
                    "amount": 1,
                    "currencyCode": "KES",
                },
                "responses": [
                    {
                        "phoneNumber": {
                            "carrierName": "Safaricom",
                            "countryCode": 254,
                            "networkCode": "Safaricom",
                            "number": "+2547xxxxxxxx",
                            "numberType": "Mobile",
                        },
                        "status": "Queued",
                        "requestId": "ATSwpid_e6968e1d3a8e95ffa3dd9454dde5d1ab",
                        "cost": {"amount": 1, "currencyCode": "KES"},
                    }
                ],
            },
            status=200,
        )
        res = service.check_sim_swap_state(["+254712345678"])
        assert res["status"] == "Processed"


if __name__ == "__main__":
    unittest.main()



import json
import warnings
from schema import And, Optional, Schema, SchemaError
from .Service import DEFAULT_TIMEOUT_S, Service, validate_phone

media_types = {
    "Image": "Image",
    "Audio": "Audio",
    "Video": "Video",
    "Sticker": "Sticker",
    "Voice": "Voice",
}

header_types = {
    "MEDIA": "MEDIA",
    "TEXT": "TEXT",
    "LOCATION": "LOCATION",
}

button_types = {
    "PHONE_NUMBER": "PHONE_NUMBER",
    "URL": "URL",
    "QUICK_REPLY": "QUICK_REPLY",
}


class WhatsappService(Service):
    def __init__(self, username, api_key):
        super(WhatsappService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = "https://chat." + self._PRODUCTION_DOMAIN
        if self._is_sandbox():
            warnings.warn(
                "Sandbox is currently not available for the Whatsapp service."
            )

    def send(
        self,
        body,
        wa_number,
        phone_number,
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        try:
            data = {
                "username": self._username,
                "waNumber": wa_number,
                "phoneNumber": phone_number,
                "body": body,
            }
            messageSchema = Schema(
                {
                    "username": And(str),
                    "waNumber": And(
                        str,
                        lambda s: validate_phone(s),
                        error=f"Invalid whatsapp number: {wa_number}",
                    ),
                    "phoneNumber": And(
                        str,
                        lambda s: validate_phone(s),
                        error=f"Invalid phone number: {phone_number}",
                    ),
                    "body": {
                        Optional("mediaType"): And(
                            str,
                            lambda s: s in media_types,
                            error=f"Invalid media type, valid types are {list(media_types.keys())}",
                        ),
                        Optional("url"): And(
                            str,
                            lambda s: "mediaType" in body,
                            error="mediaType required for url key",
                        ),
                        Optional("caption"): And(
                            str,
                            lambda s: "url" in body,
                            error="url required for caption key",
                        ),
                        Optional("message"): And(
                            str,
                            lambda s: "mediaType" not in body,
                            error="mediaType cannot be used with message key",
                        ),
                        Optional("body"): {"text": And(str)},
                        Optional("header"): {"text": And(str)},
                        Optional("footer"): {"text": And(str)},
                    },
                }
            )
            data = messageSchema.validate(data)
        except SchemaError as err:
            raise ValueError(err)

        url = self._make_url("/whatsapp/message/send")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        data = json.dumps(data)

        return self._make_request(
            url,
            "POST",
            headers=headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )

    def send_template(
        self,
        components,
        wa_number,
        name,
        language,
        category,
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        try:
            data = {
                "username": self._username,
                "waNumber": wa_number,
                "name": name,
                "language": language,
                "category": category,
                "components": components,
            }
            messageSchema = Schema(
                {
                    "username": And(str),
                    "waNumber": And(
                        str,
                        lambda s: validate_phone(s),
                        error=f"Invalid whatsapp number: {wa_number}",
                    ),
                    "name": And(str),
                    "language": And(str),
                    "category": And(str),
                    "components": {
                        Optional("header"): {
                            "type": And(
                                str,
                                lambda s: s in header_types,
                                error=f"Invalid header type, valid types are {list(header_types.keys())}",
                            ),
                            "format": And(str),
                            "text": And(str),
                            "example": {"header_text": And(list)},
                        },
                        "body": {
                            "type": And(str),
                            "text": And(str),
                            "example": {"body_text": And(list)},
                        },
                        Optional("footer"): {
                            "type": And(str),
                            "text": And(str),
                        },
                        Optional("buttons"): {
                            "type": And(
                                str,
                                lambda s: s in button_types,
                                error=f"Invalid button type, valid types are {list(button_types.keys())}",
                            ),
                            "url": And(str),
                            "text": And(str),
                            "phoneNumber": And(str),
                            "example": And(list),
                        },
                    },
                }
            )
            data = messageSchema.validate(data)
        except SchemaError as err:
            raise ValueError(err)

        url = self._make_url("whatsapp/template/send")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        data = json.dumps(data)

        return self._make_request(
            url,
            "POST",
            headers=headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )



import json
from .Service import DEFAULT_TIMEOUT_S, APIService


class TokenService(APIService):
    def __init__(self, username, api_key):
        super(TokenService, self).__init__(username, api_key)

    def generate_auth_token(self, callback=None, timeout=DEFAULT_TIMEOUT_S):
        url = self._make_url("/auth-token/generate")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        data = json.dumps({"username": self._username})

        return self._make_request(
            url,
            "POST",
            headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )



"""
Voice

call(phoneNumber: String): Initiate a phone call

fetchQueuedCalls(phoneNumber: String): Get queued calls

uploadMediaFile(phoneNumber: String, url: String): Upload voice media file

ActionBuilder: Build voice xml when callback URL receives a POST from the voice API
    say(text: String)

    play(url: String)

    getDigits(text: String, url: String, numDigits: Integer, timeout: Integer, finishOnKey: String, callbackUrl: String)

    dial(phoneNumbers: String, ringbackTone: String, record: Boolean, sequential: Boolean, callerId: String, maxDuration: Integer)

    conference()

    record()

    record(String text, URL url, int maxLength, long timeout, String finishOnKey, boolean trimSilence, boolean playBeep, URL callbackUrl)

    enqueue()

    dequeue()

    reject()

    redirect()

    build(): Finally build the xml

"""

import africastalking
import responses
import unittest
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Voice


class TestVoiceService(unittest.TestCase):
    @responses.activate
    def test_call(self):
        responses.add(
            responses.POST,
            "https://voice.africastalking.com/call",
            json={
                "entries": [
                    {
                        "phoneNumber": "+234XXXYYYZZZZ",
                        "status": "Queued",
                        "sessionId": "ATVId_abcdef",
                    }
                ],
                "errorMessage": "None",
            },
            status=200,
        )
        res = service.call(
            callFrom="+254711223355", callTo=["+254711223366", "+254711223344"]
        )
        assert res["errorMessage"] == "None"

    @responses.activate
    def test_fetch_queued_calls(self):
        responses.add(
            responses.POST,
            "https://voice.africastalking.com/queueStatus",
            json={
                "status": "Success",
                "entries": [
                    {"phoneNumber": "+254711223366", "queueName": "", "numCalls": 1}
                ],
                "errorMessage": "None",
            },
            status=200,
        )
        res = service.fetch_queued_calls(phone_number="+254711223366")

        assert res["status"] == "Success"

    @responses.activate
    def test_media_upload(self):
        responses.add(
            responses.POST,
            "https://voice.africastalking.com/mediaUpload",
            json={
                "success": True,
            },
            status=201,
        )
        res = service.media_upload(
            phone_number="+254711223355", url="https://aksalj.com"
        )
        assert res["success"]


if __name__ == "__main__":
    unittest.main()



"""
SMS

send(message: String, recipients: List<String>, senderId: Optional<String>, enqueue: Optional<Boolean>): Send a bulk message to recipients, optionally from senderId (Short Code or Alphanumeric).

sendPremium(message: String, keyword: String, linkId: String, recipients: List<String>, senderId: Optional<String>, retryDurationInHours: Optional<Integer>): Send a premium SMS

fetchMessages(lastReceivedId: Optional<Integer>): Fetch your messages

fetchSubscriptions(shortCode: String, keyword: String, lastReceivedId: Optional<Integer>): Fetch your premium subscription data

createSubscription(shortCode: String, keyword: String, phoneNumber: String): Create a premium subscription
"""

import africastalking
import unittest
import responses
from africastalking.Service import DEFAULT_TIMEOUT_S
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
token_service = africastalking.Token
service = africastalking.SMS


class TestSmsService(unittest.TestCase):
    @responses.activate
    def test_send(self):
        responses.add(
            responses.POST,
            "https://api.africastalking.com/version1/messaging",
            json={
                "SMSMessageData": {
                    "Message": "Sent to 2/2 Total Cost: KES 0.8000",
                    "Recipients": [
                        {
                            "statusCode": 101,
                            "number": "+254718769882",
                            "status": "Success",
                            "cost": "KES 0.8000",
                            "messageId": "ATPid_SampleTxnId123",
                        },
                        {
                            "statusCode": 101,
                            "number": "+254718769881",
                            "status": "Success",
                            "cost": "KES 0.8000",
                            "messageId": "ATPid_SampleTxnId1234",
                        },
                    ],
                }
            },
            status=200,
            match=[
                responses.matchers.request_kwargs_matcher(
                    {
                        "timeout": DEFAULT_TIMEOUT_S,
                    }
                )
            ],
        )
        res = service.send(
            "test_send()",
            ["+254718769882", "+254718769881"],
            enqueue=True,
            sender_id="AT2FA",
        )
        recipients = res["SMSMessageData"]["Recipients"]
        assert len(recipients) == 2
        assert recipients[0]["status"] == "Success"

    @responses.activate
    def test_heavy_single_send(self):
        count = 1000
        phone_numbers = list(
            map(lambda x: str("+254718" + str(x + count)), range(1, count))
        )
        recipients = []
        for i in range(len(phone_numbers)):
            recipients += {
                "statusCode": 101,
                "number": phone_numbers[i],
                "status": "Success",
                "cost": "KES 0.8000",
                "messageId": "ATPid_SampleTxnId123",
            }
        responses.add(
            responses.POST,
            "https://api.africastalking.com/version1/messaging",
            json={
                "SMSMessageData": {
                    "Message": f"Sent to {count}/{count} Total Cost: KES 0.8000",
                    "Recipients": recipients,
                }
            },
            status=200,
            match=[
                responses.matchers.request_kwargs_matcher(
                    {
                        "timeout": DEFAULT_TIMEOUT_S,
                    }
                )
            ],
        )

        def on_finish(error, data):
            if error:
                raise error
            recipients = data["SMSMessageData"]["Recipients"]
            assert len(recipients) <= count

        service.send(
            "test_heavy_single_send()",
            phone_numbers,
            enqueue=True,
            sender_id="AT2FA",
            callback=on_finish,
        )

    @responses.activate
    def test_send_premium(self):
        responses.add(
            responses.POST,
            "https://content.africastalking.com/version1/messaging",
            json={
                "SMSMessageData": {
                    "Message": "Sent to 2/2 Total Cost: KES 0.8000",
                    "Recipients": [
                        {
                            "statusCode": 101,
                            "number": "+254718769882",
                            "status": "Success",
                            "cost": "KES 0.8000",
                            "messageId": "ATPid_SampleTxnId123",
                        },
                        {
                            "statusCode": 101,
                            "number": "+254718769881",
                            "status": "Success",
                            "cost": "KES 0.8000",
                            "messageId": "ATPid_SampleTxnId1234",
                        },
                    ],
                }
            },
            status=200,
            match=[
                responses.matchers.request_kwargs_matcher(
                    {
                        "timeout": DEFAULT_TIMEOUT_S,
                    }
                )
            ],
        )
        res = service.send_premium(
            "test_send_premium()",
            "AT2FA",
            ["+254718769882", "+254718769881"],
            "KiKi",
            "Linky",
            retry_duration_in_hours=10,
        )
        recipients = res["SMSMessageData"]["Recipients"]
        assert len(recipients) == 2
        assert recipients[0]["status"] == "Success"

    @responses.activate
    def test_fetch_messages(self):
        responses.add(
            responses.GET,
            f"https://api.africastalking.com/version1/messaging?username={USERNAME}&lastReceivedId=0",
            json={
                "SMSMessageData": {
                    "Messages": [
                        {
                            "linkId": "SampleLinkId123",
                            "text": "Hello",
                            "to": "28901",
                            "id": 15071,
                            "date": "2018-03-19T08:34:18.445Z",
                            "from": "+254711XXXYYY",
                        }
                    ]
                }
            },
            status=200,
            match=[
                responses.matchers.request_kwargs_matcher(
                    {
                        "timeout": DEFAULT_TIMEOUT_S,
                    }
                )
            ],
        )
        res = service.fetch_messages(0)
        assert len(res) >= 0

    @responses.activate
    def test_create_safaricom_subscription(self):
        responses.add(
            responses.POST,
            "https://content.africastalking.com/version1/subscription/safaricom",
            json={
                "responseCode": "Success",
                "status": "Sent",
                "transactionId": "Tid_3886639048695350000",
                "url": "https://dcbatf2.safaricom.co.ke/v2/service/safaricom/bluu/Tid_177539048695398400?onError=https%3A%2F%2Fmidge-driven-flamingo.ngrok-free.app%2Fcontent%2Fpremium%2Fevina%2Fsubscription%3Ftype%3DERR%26rId%3DrequestId-9168bb62-3b6a-4a87-811d-4ef4c7e1e6dd",
            },
            status=200,
            match=[
                responses.matchers.request_kwargs_matcher(
                    {
                        "timeout": DEFAULT_TIMEOUT_S,
                    }
                )
            ],
        )
        res = service.create_safaricom_subscription(
            short_code="78942",
            keyword="KiKi",
            phone_number="+254718769882",
            redirect_url="http://test.test",
        )
        assert res["responseCode"] == "Success"


if __name__ == "__main__":
    unittest.main()



"""
Whatsapp

send(): Send Whatsapp message.
"""

import africastalking
import unittest
import responses
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Whatsapp


class TestWhatsappService(unittest.TestCase):
    @responses.activate
    def test_send_whatsapp(self):
        responses.add(
            responses.POST,
            "https://chat.africastalking.com/whatsapp/message/send",
            json={
                "messageId": "ATXid_91292754713251840",
                "phoneNumber": "+254700000000",
                "status": "SENT",
            },
            status=200,
        )
        res = service.send({"message": "test"}, "+254700000000", "+25471111111")
        assert res["status"] == "SENT"


if __name__ == "__main__":
    unittest.main()



"""
Token

generateAuthToken(): Generate an auth token to us for authentication instead of the API key.

"""

import africastalking
import unittest
import responses
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Token


class TestTokenService(unittest.TestCase):
    @responses.activate
    def test_generate_auth_token(self):
        responses.add(
            responses.POST,
            "https://api.africastalking.com/auth-token/generate",
            json={"lifetimeInSeconds": 3600, "token": "jriojrigj10939042904"},
            status=200,
        )
        res = service.generate_auth_token()
        assert res["lifetimeInSeconds"] == 3600


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
from setuptools import setup
import sys
import os

version = "2.0.2"

long_description = open("README.md").read()

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

setup(
    name="africastalking",
    version=version,
    packages=["africastalking"],
    description="Official Africa's Talking Python SDK",
    data_files=[("", ["README.md"])],
    license="MIT",
    author="Africa's Talking",
    install_requires=["requests>=v2.30.0", "schema>=0.6.7", "responses>=0.25.8"],
    python_requires=">=3.8",
    author_email="info@africastalking.com",
    url="https://github.com/AfricasTalkingLtd/africastalking-python",
    download_url="https://codeload.github.com/AfricasTalkingLtd/africastalking-python/tar.gz/"
    + version,
    keywords="ussd voice sms mpesa sender_id airtime africastalking mobile_data sim_swap whatsapp",
    classifiers=[],
    long_description=long_description,
    long_description_content_type="text/markdown",
)



"""
Application

fetchApplicationData(): Fetch app info i.e. balance.
"""

import africastalking
import unittest
import responses
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Application


class TestApplicationService(unittest.TestCase):
    @responses.activate
    def test_fetch_account(self):
        responses.add(
            responses.GET,
            f"https://api.africastalking.com/version1/user?username={USERNAME}",
            json={"userData": {"balance": "KES 1785.50"}},
            status=200,
        )
        res = service.fetch_application_data()
        assert res["userData"]["balance"] is not None


if __name__ == "__main__":
    unittest.main()



import africastalking
import unittest
import responses
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.MobileData


class TestMobileDataService(unittest.TestCase):
    @responses.activate
    def test_mobile_data(self):
        try:
            responses.add(
                responses.POST,
                "https://bundles.africastalking.com/mobile/data/request",
                json={
                    "entries": [
                        {
                            "phoneNumber": "+254714978532",
                            "provider": "Safaricom",
                            "status": "Queued",
                            "transactionId": "ATPid_8df0ca90660d981ff9059454672496c2",
                            "value": "KES 20.0000",
                        }
                    ]
                },
                status=200,
            )
            recipient = {
                "phoneNumber": "+254711223344",
                "quantity": 10,
                "unit": "GB",
                "validity": "Month",
                "metadata": {"some": "information"},
            }
            res = service.send(product_name="TestProduct", recipients=[recipient])
            assert res["entries"][0]["status"] is not None
        except africastalking.Service.AfricasTalkingException:
            assert True



from .Service import APIService, DEFAULT_TIMEOUT_S, validate_phone
from schema import Schema, And, Optional, SchemaError


class SMSService(APIService):
    def __init__(self, username, api_key):
        super(SMSService, self).__init__(username, api_key)

    def _init_service(self):
        super(SMSService, self)._init_service()
        self._baseUrl = self._baseUrl + "/version1"
        if self._contentUrl:
            self._contentUrl = self._contentUrl + "/version1"

    def send(
        self,
        message,
        recipients,
        sender_id=None,
        enqueue=False,
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError("Invalid phone number: " + phone)

        url = self._make_url("/messaging")
        data = {
            "username": self._username,
            "to": ",".join(recipients),
            "message": message,
            "bulkSMSMode": 1,
        }

        if sender_id is not None:
            data["from"] = sender_id

        if enqueue:
            data["enqueue"] = 1

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )

    def send_premium(
        self,
        message,
        short_code,
        recipients,
        keyword=None,
        link_id=None,
        retry_duration_in_hours=None,
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError("Invalid phone number: " + phone)

        url = self._make_url("/messaging", content=True)
        data = {
            "username": self._username,
            "to": ",".join(recipients),
            "from": short_code,
            "message": message,
            "bulkSMSMode": 0,
        }

        if link_id is not None:
            data["linkId"] = link_id

        if keyword is not None:
            data["keyword"] = keyword

        if retry_duration_in_hours is not None:
            data["retryDurationInHours"] = retry_duration_in_hours

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )

    def fetch_messages(
        self, last_received_id=None, callback=None, timeout=DEFAULT_TIMEOUT_S
    ):
        url = self._make_url("/messaging")
        params = {"username": self._username}

        if last_received_id is not None:
            params["lastReceivedId"] = last_received_id

        return self._make_request(
            url,
            "GET",
            headers=self._headers,
            params=params,
            data=None,
            callback=callback,
            timeout=timeout,
        )

    def create_safaricom_subscription(
        self,
        short_code,
        keyword,
        phone_number=None,
        request_id=None,
        redirect_url=None,
        source_ip=None,
        user_agent=None,
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        try:
            data = {
                "username": self._username,
                "shortCode": short_code,
                "keyword": keyword,
                "requestId": request_id,
                "redirectUrl": redirect_url,
                "sourceIp": source_ip,
                "userAgent": user_agent,
                "phoneNumber": phone_number,
            }
            for k, v in list(data.items()):
                if v is None:
                    del data[k]

            messageSchema = Schema(
                {
                    "username": And(str),
                    "shortCode": And(str),
                    "keyword": And(str),
                    "redirectUrl": And(str),
                    Optional("userAgent"): And(str),
                    Optional("sourceIp"): And(str),
                    Optional("phoneNumber"): And(
                        str,
                        lambda s: validate_phone(s),
                        error=f"Invalid phone number: {phone_number}",
                    ),
                    Optional("requestId"): And(str),
                }
            )
            data = messageSchema.validate(data)
        except SchemaError as err:
            raise ValueError(err)

        url = self._make_url("/subscription/safaricom", content=True)

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            data=data,
            params=None,
            callback=callback,
            timeout=timeout,
        )

    def send_hashed(
        self,
        message,
        masked_number,
        sender_id,
        telco="Safaricom",
        callback=None,
        timeout=DEFAULT_TIMEOUT_S,
    ):
        url = self._make_url("/messaging/bulk")
        data = {
            "username": self._username,
            "maskedNumber": masked_number,
            "telco": telco,
            "message": message,
            "senderId": sender_id,
            "phoneNumbers": [],
        }

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
            timeout=timeout,
        )
